﻿#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "nrf_assert.h"
#include "nrf_drv_wdt.h"
#include "ble_hci.h"
#include "ble_nus.h"
#include "ble_advdata.h"
#include "ble_conn_params.h"
#include "nrf_sdh.h"
#include "fds.h"
#include "app_uart.h"
#include "app_fifo.h"
#include "app_timer.h"
#include "app_gpiote.h"
#include "app_scheduler.h"
#include "app_util_platform.h"
#include "nrf_ble_gatt.h"
#include "app_button.h"
#include "minew_board.h"
#include "command_def.h"
#include "app_simple_timer.h"
#include "app_util.h"
#include "nrf_drv_power.h"

// #define NRF_LOG_MODULE_NAME main_
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
// NRF_LOG_MODULE_REGISTER();

#define CONN_CFG_TAG                    1 

#define SCHED_MAX_EVENT_DATA_SIZE       8                                                /**< SCHED事件事件中可放入变量的最大值. */
#define SCHED_QUEUE_SIZE                8                                                /**< 最多SCHED事件挂起数量. */

#define DEAD_BEEF                       0xDEADBEEF                                       /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

#define UART_TX_BUF_SIZE                256                                              /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE                4                                                /**< UART RX buffer size. */

#define LED_HIGH_PULSE_TIME             200
#define LED_LOW_PULSE_TIME              (1000 - LED_HIGH_PULSE_TIME)
#define WDT_TIMEOUT_TIME                4000
#define BUTTON_DETECTION_DELAY          APP_TIMER_TICKS(20)                              /**< 按键消抖延时时间. */
#define FILE_ID_MS_HANDLE               0x0001
#define REC_KEY_MS_HANDLE               0x0201

/**
 * #brief 串口接收模块工作状态. 
 */
typedef enum {
    UART_REC_IDLE,              /**< 串口接收模块当前没有工作. */   
    UART_REC_DATA_PASS,         /**< 串口接收模块工作在透传状态下. */
    UART_REC_DATA_PASS_WAIT,    /**< 串口接收模块工作在透传状态下, 等待协议栈ble buffer空闲. */
    UART_REC_CMD_ANALY          /**< 串口接收模块工作在命令状态下. */
} uart_rec_states_t;


typedef enum {
    UART_CMD_ADV_NAME,    /**< 广播名称. */
    UART_CMD_ADV_INTE,    /**< 广播间隔. */
    UART_CMD_CON_INTE,    /**< 连接间隔. */
    UART_CMD_TX_POER,     /**< 发射功率. */
    UART_CMD_UART_BPS,    /**< 串口波特率. */
    UART_CMD_MAC_ADD,     /**< mad地址. */
    UART_CMD_VERSON,      /**< 版本号. */
    UART_CMD_RESTE,       /**< 复位. */
    UART_CMD_ADV_DATA,    /**< 广播数据. */
    UART_CMD_IDEN_NUM,    /**< 产品识别码. */
    UART_CMD_RESTORE,     /**< 恢复出厂设置. */
    UART_CMD_UUID,        /**< uuid. */
    UART_CMD_ERROR
} uart_cmd_t;


typedef enum {
    UART_REPLY_OK,
    UART_REPLY_ERROR,
    UART_REPLY_MAC,
    UART_REPLY_VERSON,
} uart_reply_t;


/**
 * #简介 LED模块工作状态.
 */
typedef enum {
    LED_IDLE,
    LED_SLEEP,
    LED_BLE_CONNECT,
    LED_BLE_ADVERTISING
} app_led_mode_t;


/**
 * #brief ...
 */
typedef struct {
    uint8_t    data[BLE_NUS_MAX_DATA_LEN];
    int        size;
} temp_array_t;

static nrf_drv_wdt_channel_id    m_channel_id;     
static cmd_collection_t          m_cmd_collection;                                         /**< BLE uart command collection. */
static uart_rec_states_t         m_uart_rec_states = UART_REC_IDLE;                        /**< Uart receive process states. */
static temp_array_t              m_temp_array;                                             /**< BLE notify data array. */
static app_fifo_t                m_uart_rec_fifo;                                          /**< Uart rec fifo use for ble notify data array. */
static uart_cmd_t                m_uart_cmd_ty;
static uint8_t                   m_rec_buf[4096];                                          /**< Uart rec fifo memery use for ble notify data array. */
static uint16_t                  m_conn_handle            = BLE_CONN_HANDLE_INVALID;       /**< Handle of the current connection. */
static uint16_t                  m_ble_nus_max_data_len   = BLE_GATT_ATT_MTU_DEFAULT - 3;  /**< Maximum length of data (in bytes) that can be transmitted to the peer by the Nordic UART service module. */
static uint8_t                   m_version[VERSION_LENTH] = DEFAULT_VERSION;               /**< Firmware version . */ 
static bool                      m_fds_init_flag          = false;                         /**< 初始化成功标志 . */
static uint8_t                   m_update_in_queue        = 0;

extern uint16_t nus_service_uuid;
extern uint16_t nus_rx_char_uuid;
extern uint16_t nus_tx_char_uuid;

BLE_NUS_DEF(m_nus);                                                                 /**< BLE NUS service instance. */
NRF_BLE_GATT_DEF(m_gatt);                                                           /**< GATT module instance. */
APP_TIMER_DEF(wstates_led_time_id);
APP_TIMER_DEF(wdt_time_id);
APP_TIMER_DEF(reset_time_id);
APP_TIMER_DEF(uart_clear_id);


//! 函数声明
static void uart_rec_process(void * p_context, uint16_t data_length);
static void advertising_init(void);
static void advertising_start(void);
static void default_param_get(void);
static void ble_data_notify_handler(void * p_context);
static void uart_init(void);


/**
 * #brief Function for determine whether the device is connected.
 *
 * #return true   if device is connected.
 * #return false  if device is not connected.
 */
static inline bool is_device_connect(void) {
    return (m_conn_handle == BLE_CONN_HANDLE_INVALID)? false : true;
}


/**
 * #简介 数据翻转.
 * #说明 传入的size必须为偶数，否则将自动忽略最后一位.
 */
static inline void array_reserve(uint8_t * p_data, uint16_t length)
{
    if (length <= 1) {
        return;
    }

    if ((length % 2) != 0) {
        length -= 1;
    }

    uint8_t tmp_exchange;
    for (uint8_t tmp_index = 0; tmp_index < (length / 2); tmp_index++) {
        tmp_exchange                   = p_data[tmp_index];
        p_data[tmp_index]              = p_data[length - tmp_index - 1];
        p_data[length - tmp_index - 1] = tmp_exchange;
    }
}


/**
 * #简介 LED工作模式设置.
 */
static void app_led_handle(app_led_mode_t led_op, uint8_t pin_no)
{
    static bool stmp_led_states = false;

    switch (led_op)
    {
        case LED_SLEEP:
            //! 睡眠状态时WSTATES引脚为低电平
            APP_ERROR_CHECK(app_timer_stop(wstates_led_time_id));
            nrf_gpio_pin_clear(pin_no);
            break;

        case LED_BLE_CONNECT:
            //! 连接状态时WSTATES引脚为高电平
            APP_ERROR_CHECK(app_timer_stop(wstates_led_time_id));
            nrf_gpio_pin_set(pin_no);
            break;

        case LED_BLE_ADVERTISING:
            //! 广播状态时WSTATES引脚为200ms高电平,周期1s
            nrf_gpio_pin_clear(pin_no);
            stmp_led_states = false;
            APP_ERROR_CHECK(app_timer_start(wstates_led_time_id, APP_TIMER_TICKS(LED_LOW_PULSE_TIME), &stmp_led_states));
            break;

        default:
            break;
    }
}


/**
 * #brief ...
 */
static inline void uart_put_string(uint8_t * p_data, uint8_t len) {

    for (uint8_t i = 0; i < len; i++) {
        APP_ERROR_CHECK(app_uart_put(p_data[i]));
    }
}


/**
 * #brief ...
 */
static inline void uart_reply_tail(void) {
    uint8_t tmp_tail[] = { '\r', '\n', 0x00 };
    uart_put_string(tmp_tail, sizeof(tmp_tail));
}


/**
 * #brief ...
 */
static inline void uart_reply(bool is_success)
{
    char tmp_reply[32] = { 0 };
    if (is_success == true) {
        memcpy(tmp_reply, (uint8_t *)COMMAND_REPLY_CONFIRM, strlen(COMMAND_REPLY_CONFIRM));
    } else {
        memcpy(tmp_reply, (uint8_t *)COMMAND_REPLY_ERR, strlen(COMMAND_REPLY_ERR));
    }
    uart_put_string((uint8_t *)tmp_reply, strlen(tmp_reply));
    uart_reply_tail();
}


/**
 * #brief ...
 */
static inline void uart_rec_process_clr(void) 
{   
    m_uart_rec_states /**#StatesChange.*/ = UART_REC_IDLE;
    memset(&m_temp_array, 0, sizeof(m_temp_array));

    APP_ERROR_CHECK(app_simple_timer_stop());
    APP_ERROR_CHECK(app_fifo_flush(&m_uart_rec_fifo));
    APP_ERROR_CHECK(app_uart_flush());
}


/**
 * #brief Function for assert macro callback.
 *
 * #details This function will be called in case of an assert in the SoftDevice.
 *
 * #param[in] line_num    Line number of the failing ASSERT call.
 * #param[in] p_file_name File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name){
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}


/**
 * #brief Function for the GAP initialization.
 *
 * #details This function will set up all the necessary GAP (Generic Access Profile) parameters of
 *          the device. It also sets the permissions and appearance.
 */
static void gap_params_init(void)
{
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);
    APP_ERROR_CHECK(sd_ble_gap_device_name_set(&sec_mode,
                                               m_cmd_collection.adv_name.data_array,
                                               m_cmd_collection.adv_name.data_length));

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MSEC_TO_UNITS(m_cmd_collection.con_interval_level * 10, UNIT_1_25_MS);
    gap_conn_params.max_conn_interval = MSEC_TO_UNITS(m_cmd_collection.con_interval_level * 10 + 10, UNIT_1_25_MS);
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    APP_ERROR_CHECK(sd_ble_gap_ppcp_set(&gap_conn_params));
}


/**
 * #brief Function for handling the data from the Nordic UART Service.
 *
 * #details This function will process the data received from the Nordic UART BLE Service and send
 *          it to the UART module.
 *
 * #param[in] p_nus    Nordic UART Service structure.
 * #param[in] p_data   Data to be send to UART module.
 * #param[in] length   Length of the data.
 */
static void nus_data_handler(ble_nus_evt_t * p_evt)
{
    uint32_t err_code;

    switch (p_evt->type) {
        case BLE_NUS_EVT_RX_DATA:

            NRF_LOG_DEBUG("Received data from BLE NUS. Writing data on UART.");
            NRF_LOG_HEXDUMP_DEBUG(p_evt->params.rx_data.p_data, p_evt->params.rx_data.length);

            for (uint32_t i = 0; i < p_evt->params.rx_data.length; i++)
            {
                do
                {
                    err_code = app_uart_put(p_evt->params.rx_data.p_data[i]);
                    if ((err_code != NRF_SUCCESS) && (err_code != NRF_ERROR_BUSY)) {
                        NRF_LOG_ERROR("Failed receiving NUS message. Error 0x%x. ", err_code);
                        APP_ERROR_CHECK(err_code);
                    }
                } while (err_code == NRF_ERROR_BUSY);
            }
            if (p_evt->params.rx_data.p_data[p_evt->params.rx_data.length-1] == '\r')
            {
                while (app_uart_put('\n') == NRF_ERROR_BUSY);
            }
            break;

        case BLE_NUS_EVT_TX_RDY:
            if (m_uart_rec_states == UART_REC_DATA_PASS_WAIT) {
                NRF_LOG_WARNING("uart reboot\r\n");
                ble_data_notify_handler(NULL);
            }
            break;
        default :   
            break;
    }
}


/**
 * #brief Function for initializing services that will be used by the application.
 */
static void services_init(void)
{
    ble_nus_init_t nus_init;
    memset(&nus_init, 0, sizeof(nus_init));
    nus_service_uuid = m_cmd_collection.nus_uuid;
    nus_tx_char_uuid = m_cmd_collection.nus_tx_uuid;
    nus_rx_char_uuid = m_cmd_collection.nus_rx_uuid;

    nus_init.data_handler = nus_data_handler;
    APP_ERROR_CHECK(ble_nus_init(&m_nus, &nus_init));
}


/**
 * #brief Function for handling an event from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED) {
        APP_ERROR_CHECK(sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE));
    }else{
        app_led_handle(LED_BLE_CONNECT, WSTATES_PIN_NUMBER);
    }
}


/**
 * #brief Function for handling errors from the Connection Parameters module.
 *
 * #param[in] nrf_error  Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error){
    APP_ERROR_HANDLER(nrf_error);
}


/**
 * #简介 当前是否有update操作还未停止.
 * 
 * #返回true表示没有update操作在进行中.
 */
bool app_flash_update_done(void) {
	return ((m_update_in_queue == 0)? true : false);
}


/**
 * #brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)
{
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    APP_ERROR_CHECK(ble_conn_params_init(&cp_init));
}


/**
 * @brief Function for handling BLE events.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 * @param[in]   p_context   Unused.
 */
static void ble_evt_handler(ble_evt_t const *p_ble_evt, void *p_context)
{
    ret_code_t err_code;
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            NRF_LOG_INFO("connected\r\n");
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            uart_rec_process_clr();
            break;

        case BLE_GAP_EVT_DISCONNECTED:
            NRF_LOG_INFO("Disconnected\r\n");
            NRF_LOG_RAW_INFO("disconnected reason %x\r\n",
                            p_ble_evt->evt.gap_evt.params.disconnected.reason);
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
            uart_rec_process_clr();

            advertising_init();
            advertising_start();
            break;
#ifndef S140
        case BLE_GAP_EVT_PHY_UPDATE_REQUEST:
        {
            NRF_LOG_DEBUG("PHY update request.");
            ble_gap_phys_t const phys =
            {
                .rx_phys = BLE_GAP_PHY_AUTO,
                .tx_phys = BLE_GAP_PHY_AUTO,
            };
            err_code = sd_ble_gap_phy_update(p_ble_evt->evt.gap_evt.conn_handle, &phys);
            APP_ERROR_CHECK(err_code);
        } break;
#endif

        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
            // Pairing not supported
            err_code = sd_ble_gap_sec_params_reply(m_conn_handle, BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, NULL, NULL);
            APP_ERROR_CHECK(err_code);
            break;
#if !defined (S112)
         case BLE_GAP_EVT_DATA_LENGTH_UPDATE_REQUEST:
        {
            ble_gap_data_length_params_t dl_params;

            // Clearing the struct will effectivly set members to @ref BLE_GAP_DATA_LENGTH_AUTO
            memset(&dl_params, 0, sizeof(ble_gap_data_length_params_t));
            err_code = sd_ble_gap_data_length_update(p_ble_evt->evt.gap_evt.conn_handle, &dl_params, NULL);
            APP_ERROR_CHECK(err_code);
        } break;
#endif //!defined (S112)
        case BLE_GATTS_EVT_SYS_ATTR_MISSING:
            // No system attributes have been stored.
            err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0, 0);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTC_EVT_TIMEOUT:
            // Disconnect on GATT Client timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gattc_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_TIMEOUT:
            // Disconnect on GATT Server timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gatts_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_EVT_USER_MEM_REQUEST:
            err_code = sd_ble_user_mem_reply(p_ble_evt->evt.gattc_evt.conn_handle, NULL);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_RW_AUTHORIZE_REQUEST:
        {
            ble_gatts_evt_rw_authorize_request_t  req;
            ble_gatts_rw_authorize_reply_params_t auth_reply;

            req = p_ble_evt->evt.gatts_evt.params.authorize_request;

            if (req.type != BLE_GATTS_AUTHORIZE_TYPE_INVALID)
            {
                if ((req.request.write.op == BLE_GATTS_OP_PREP_WRITE_REQ)     ||
                    (req.request.write.op == BLE_GATTS_OP_EXEC_WRITE_REQ_NOW) ||
                    (req.request.write.op == BLE_GATTS_OP_EXEC_WRITE_REQ_CANCEL))
                {
                    if (req.type == BLE_GATTS_AUTHORIZE_TYPE_WRITE)
                    {
                        auth_reply.type = BLE_GATTS_AUTHORIZE_TYPE_WRITE;
                    }
                    else
                    {
                        auth_reply.type = BLE_GATTS_AUTHORIZE_TYPE_READ;
                    }
                    auth_reply.params.write.gatt_status = APP_FEATURE_NOT_SUPPORTED;
                    err_code = sd_ble_gatts_rw_authorize_reply(p_ble_evt->evt.gatts_evt.conn_handle,
                                                               &auth_reply);
                    APP_ERROR_CHECK(err_code);
                }
            }
        } break; // BLE_GATTS_EVT_RW_AUTHORIZE_REQUEST

        default:
            // No implementation needed.
            break;
    }
}


/**
 * #brief Function for the SoftDevice initialization.
 *
 * #details This function initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    APP_ERROR_CHECK(nrf_sdh_enable_request());
  
    /* Configure the BLE stack using the default settings. */
    /* Fetch the start address of the application RAM. */
    uint32_t ram_start = 0;
  
    APP_ERROR_CHECK(nrf_sdh_ble_default_cfg_set(APP_BLE_CONN_CFG_TAG, &ram_start));

    /* Enable BLE stack. */
    APP_ERROR_CHECK(nrf_sdh_ble_enable(&ram_start));

    /* Register a handler for BLE events. */
    NRF_SDH_BLE_OBSERVER(m_ble_observer, APP_BLE_OBSERVER_PRIO, ble_evt_handler, NULL);
}


/**
 * @brief Function for handling events from the GATT library.
 */
void gatt_evt_handler(nrf_ble_gatt_t * p_gatt, nrf_ble_gatt_evt_t const * p_evt)
{
    if ((m_conn_handle == p_evt->conn_handle) && (p_evt->evt_id == NRF_BLE_GATT_EVT_ATT_MTU_UPDATED)) {
        m_ble_nus_max_data_len = p_evt->params.att_mtu_effective - OPCODE_LENGTH - HANDLE_LENGTH;
        NRF_LOG_INFO("Data len is set to 0x%X(%d)", m_ble_nus_max_data_len, m_ble_nus_max_data_len);
    }
    NRF_LOG_INFO("ATT MTU exchange completed. central 0x%x peripheral 0x%x",
                  p_gatt->att_mtu_desired_central,
                  p_gatt->att_mtu_desired_periph);
}


/**
 * @brief Function for initializing the GATT library. 
 */
void gatt_init(void)
{
    APP_ERROR_CHECK(nrf_ble_gatt_init(&m_gatt, gatt_evt_handler));
    NRF_LOG_DEBUG("nrf_ble_gatt_init\r\n");

    APP_ERROR_CHECK(nrf_ble_gatt_att_mtu_periph_set(&m_gatt,43));
    NRF_LOG_DEBUG("nrf_ble_gatt_att_mtu_periph_set\r\n");
}


/**
 * #brief Function for store data to flash.
 */
static void storage_data_handler(void * p_context, uint16_t length)
{
    uint32_t err_code;

    fds_record_t        record;
    fds_record_desc_t   record_desc;
    fds_find_token_t    ftok;

    // Set up record.
    record.file_id                  = FILE_ID_MS_HANDLE;
    record.key                      = REC_KEY_MS_HANDLE;
    record.data.p_data            = &m_cmd_collection;
    record.data.length_words      = BYTES_TO_WORDS(sizeof(m_cmd_collection));
    
	memset(&ftok, 0x00, sizeof(fds_find_token_t));
	err_code = fds_record_find(FILE_ID_MS_HANDLE, REC_KEY_MS_HANDLE, &record_desc, &ftok);
	if (err_code == FDS_SUCCESS) {
        NRF_LOG_WARNING("fds find req\r\n");

		//! Update process. //////////////////////////////
		err_code = fds_record_update(&record_desc, &record);
		if (err_code == FDS_SUCCESS) {
            m_update_in_queue += 1;
            NRF_LOG_WARNING("fds update req\r\n");

		} else if (err_code == FDS_ERR_NO_SPACE_IN_FLASH) {
			NRF_LOG_WARNING("[%04d] FDS_ERR_NO_SPACE_IN_FLASH %x, %x.\r\n", __LINE__, FILE_ID_MS_HANDLE, REC_KEY_MS_HANDLE);
			if (fds_gc() == FDS_ERR_NO_SPACE_IN_QUEUES) {
				NRF_LOG_WARNING("[%04d] FDS gc ope is cancle.\r\n", __LINE__);
			}
		} else {
			APP_ERROR_CHECK(err_code);
		}
		//////////////////////////////////////////////////
	} 

    else 
    if (err_code == FDS_ERR_NOT_FOUND) {

		//! Write process. ///////////////////////////////
		NRF_LOG_WARNING("Record not found. %x, %x.\r\n", FILE_ID_MS_HANDLE, REC_KEY_MS_HANDLE);
		err_code = fds_record_write(&record_desc, &record);
		if (err_code == FDS_SUCCESS){
            NRF_LOG_WARNING("write req.\r\n");
        }

        else
        if (err_code == FDS_ERR_NO_SPACE_IN_FLASH) {
			if (fds_gc() == FDS_ERR_NO_SPACE_IN_QUEUES) {
				NRF_LOG_WARNING("[%04d] FDS gc ope is cancle.\r\n", __LINE__);
			}
		} else {
			APP_ERROR_CHECK(err_code);
		}
		//////////////////////////////////////////////////
	} 
    
    else {
		APP_ERROR_CHECK(err_code);
	}
}


/**
 * @brief 串口命令回复
 * 
 * @note 通过判断不同的状态，从串口输出相应状态
 */
static inline void mt_uart_reply(uart_reply_t uart_reply_states)
{
    ble_gap_addr_t  tmp_mac_data;
    uint8_t tmp_reply_len = 0;
    static uint8_t tmp_reply[] ={0};
    static uint8_t tmp_tail[3] = {0x0d, 0x0a, 0x00};

    switch (uart_reply_states)
    {
        case UART_REPLY_OK:
            // NRF_LOG_INFO("uart send ok %d\r\n ", strlen(COMMAND_REPLY_CONFIRM));
            memcpy(tmp_reply, (uint8_t *)COMMAND_REPLY_CONFIRM, strlen(COMMAND_REPLY_CONFIRM));
            tmp_reply_len = strlen(COMMAND_REPLY_CONFIRM);
            NRF_LOG_RAW_INFO ("cmd send ok\r\n");
            break;

        case UART_REPLY_ERROR:
            memcpy(tmp_reply, (uint8_t *)COMMAND_REPLY_ERR, strlen(COMMAND_REPLY_ERR));
            tmp_reply_len = strlen(COMMAND_REPLY_ERR);
            NRF_LOG_RAW_INFO ("cmd send erp\r\n");
            break;

        case UART_REPLY_MAC:
        {
            
            APP_ERROR_CHECK(sd_ble_gap_addr_get(&tmp_mac_data));
            memcpy(tmp_reply, (uint8_t *)CMD_MAC_REPLY, COMMAND_MIN_LEN);
            array_reserve(tmp_mac_data.addr, BLE_GAP_ADDR_LEN);
            NRF_LOG_HEXDUMP_INFO(tmp_mac_data.addr, BLE_GAP_ADDR_LEN);
            memcpy(tmp_reply + COMMAND_MIN_LEN, tmp_mac_data.addr, BLE_GAP_ADDR_LEN);
            tmp_reply_len = BLE_GAP_ADDR_LEN + COMMAND_MIN_LEN;
        }
            break;

        case UART_REPLY_VERSON:
            memcpy(tmp_reply, (uint8_t *)CMD_VERSON_REPLY, COMMAND_MIN_LEN);
            memcpy(tmp_reply + COMMAND_MIN_LEN, (uint8_t *)DEFAULT_VERSION, VERSION_LENTH);
            tmp_reply_len = COMMAND_MIN_LEN + VERSION_LENTH;
            break;
        default:
            return;
    }
    memcpy(tmp_reply + tmp_reply_len, tmp_tail, 3);
    uart_put_string(tmp_reply, tmp_reply_len + 3);
}


bool mt_uart_cmd_type_verify(uint8_t *p_data)
{
    if (memcmp(p_data, (uint8_t *)CMD_DEVICE_NAME, CMD_TYPE_LEN) == 0) {
        m_uart_cmd_ty = UART_CMD_ADV_NAME;

    } else if (memcmp(p_data, (uint8_t *)CMD_ADV_INTERVA, CMD_TYPE_LEN) == 0) {
        m_uart_cmd_ty = UART_CMD_ADV_INTE;

    } else if (memcmp(p_data, (uint8_t *)CMD_CON_INTERVA, CMD_TYPE_LEN) == 0) {
        m_uart_cmd_ty = UART_CMD_CON_INTE;

    } else if (memcmp(p_data, (uint8_t *)CMD_DEV_TX_POWER, CMD_TYPE_LEN) == 0) {
        m_uart_cmd_ty = UART_CMD_TX_POER;

    } else if (memcmp(p_data, (uint8_t *)CMD_UART_BOUNDE, CMD_TYPE_LEN) == 0) {
        m_uart_cmd_ty = UART_CMD_UART_BPS;

    } else if (memcmp(p_data, (uint8_t *)CMD_DEVICE_MAC, CMD_TYPE_LEN) == 0) {
        m_uart_cmd_ty = UART_CMD_MAC_ADD;

    } else if (memcmp(p_data, (uint8_t *)CMD_VERSION_NUM, CMD_TYPE_LEN) == 0) { 
        m_uart_cmd_ty = UART_CMD_VERSON;

    } else if (memcmp(p_data, (uint8_t *)CMD_DEVICE_RESET, CMD_TYPE_LEN + 3) == 0) {
        m_uart_cmd_ty = UART_CMD_RESTE;

    } else if (memcmp(p_data, (uint8_t *)CMD_ADV_DATA, CMD_TYPE_LEN) == 0) {
        m_uart_cmd_ty = UART_CMD_ADV_DATA;

    } else if (memcmp(p_data, (uint8_t *)CMD_DEVICE_IDEN, CMD_TYPE_LEN) == 0) {
        m_uart_cmd_ty = UART_CMD_IDEN_NUM;

    } else if (memcmp(p_data, (uint8_t *)CMD_DEVICE_RESTORE, CMD_TYPE_LEN + 3) == 0) {
        m_uart_cmd_ty = UART_CMD_RESTORE;

    } else if (memcmp(p_data, (uint8_t *)CMD_DEVICE_UUID, CMD_TYPE_LEN) == 0) {
        m_uart_cmd_ty = UART_CMD_UUID;
    } else {
        m_uart_cmd_ty = UART_CMD_ERROR;
        return false;
    }
    return true;
}


void mt_uart_cmd_go_to(uint8_t * p_data, uint8_t p_len)
{
    NRF_LOG_RAW_INFO("into mt_uart_cmd_go_to p_len =%d \r\n", p_len);
    NRF_LOG_HEXDUMP_INFO(p_data, p_len);
    ASSERT(p_data != NULL);
    // m_uart_rec_states *#StatesChange. = UART_REC_CMD_ANALY;

    bool mt_reply_ok_flag = false;
    bool mt_flash_is_change = false;
    bool mt_udef_change_flag = false;

    switch (m_uart_cmd_ty) {
        case UART_CMD_ADV_NAME:
            if (p_len >= CMD_DEV_ADV_DA_LN_MIN && 
                p_len <= CMD_DEV_NAME_LEN ) {
                if (p_len ==  m_cmd_collection.adv_name.data_length) {
                    if (memcmp(&m_cmd_collection.adv_name.data_array, p_data, p_len) != 0){
                        mt_udef_change_flag = true;
                    }
                } else {
                    mt_udef_change_flag = true;
                }
            }

            if (mt_udef_change_flag) {
                memset(m_cmd_collection.adv_name.data_array, 0, CMD_DEV_NAME_LEN);
                memcpy(&m_cmd_collection.adv_name.data_array, p_data, p_len);
                NRF_LOG_RAW_INFO("cmd dev name set\r\n");
                m_cmd_collection.adv_name.data_length = p_len;
                mt_reply_ok_flag = true;
                mt_flash_is_change = true;
            }
          
            break;    

        case UART_CMD_ADV_INTE:
            if (p_data[0] >= CMD_DEV_ADV_INTE_MIN && 
                p_data[0] <= CMD_DEV_ADV_INTE_MAX &&
                p_len == INTE_POW_BPS_DATA_LEN &&
                memcmp(&m_cmd_collection.adv_interval_level, &(p_data[0]), INTE_POW_BPS_DATA_LEN) != 0){

                memcpy(&m_cmd_collection.adv_interval_level, &(p_data[0]), INTE_POW_BPS_DATA_LEN);
                NRF_LOG_RAW_INFO("cmd adv interval is %dms", m_cmd_collection.adv_interval_level *100);
                mt_reply_ok_flag = true;
                mt_flash_is_change = true;
            }
            break;    

        case UART_CMD_CON_INTE:
            if (p_data[0] >= CMD_DEV_CONN_INTE_MIN && 
                p_data[0] <= CMD_DEV_CONN_INTE_MAX &&
                p_len == INTE_POW_BPS_DATA_LEN &&
                memcmp(&m_cmd_collection.con_interval_level, &(p_data[0]), INTE_POW_BPS_DATA_LEN) != 0) {

                memcpy(&m_cmd_collection.con_interval_level, &(p_data[0]), INTE_POW_BPS_DATA_LEN);
                NRF_LOG_RAW_INFO("cmd conn interval is %dms", m_cmd_collection.con_interval_level *100);
                mt_reply_ok_flag = true;
                mt_flash_is_change = true;
            }
            break;    

        case UART_CMD_TX_POER:
            if (p_data[0] >= CMD_DEV_TX_POWER_MIN && 
                p_data[0] <= CMD_DEV_TX_POWER_MAX &&
                p_len == INTE_POW_BPS_DATA_LEN &&
                memcmp(&m_cmd_collection.tx_power, &(p_data[0]), INTE_POW_BPS_DATA_LEN) != 0) {

                memcpy(&m_cmd_collection.tx_power, &(p_data[0]), INTE_POW_BPS_DATA_LEN);
                NRF_LOG_RAW_INFO("cmd con interval is %d", m_cmd_collection.tx_power );
                mt_reply_ok_flag = true;
                mt_flash_is_change = true;
            }
            break;    

        case UART_CMD_UART_BPS:
            if (p_data[0] >= CMD_DEV_UART_BPS_MIN && 
                p_data[0] <= CMD_DEV_UART_BPS_MAX &&
                p_len == INTE_POW_BPS_DATA_LEN &&
                memcmp(&m_cmd_collection.uart_baudrate, &(p_data[0]), INTE_POW_BPS_DATA_LEN) != 0) {

                memcpy(&m_cmd_collection.uart_baudrate, &(p_data[0]), INTE_POW_BPS_DATA_LEN);
                NRF_LOG_RAW_INFO("cmd uart bound is %d", m_cmd_collection.uart_baudrate );
                mt_reply_ok_flag = true;
                mt_flash_is_change = true;

            }
            break;   

        case UART_CMD_ADV_DATA:
            if (p_len >= CMD_DEV_ADV_DA_LN_MIN && 
                p_len <= CMD_DEV_ADV_DA_LN_MAX ) {
                if (p_len ==  m_cmd_collection.adv_data.data_length) {
                    if (memcmp(&m_cmd_collection.adv_data.data_array, p_data, p_len) != 0){
                        mt_udef_change_flag = true;
                    }
                } else {
                    mt_udef_change_flag = true;
                }
            }

            if (mt_udef_change_flag) {
                memset(m_cmd_collection.adv_data.data_array, 0, CMD_DEV_ADV_DA_LN_MAX);
                memcpy(&m_cmd_collection.adv_data.data_array, p_data, p_len);
                m_cmd_collection.adv_data.data_length = p_len;
                NRF_LOG_RAW_INFO("cmd adv data is ");
                NRF_LOG_HEXDUMP_INFO(m_cmd_collection.adv_data.data_array, m_cmd_collection.adv_data.data_length);
                mt_reply_ok_flag = true;
                mt_flash_is_change = true;
            }

            break;    

        case UART_CMD_UUID:
            if (p_len == CMD_DEV_UUID_LEN &&
                (m_cmd_collection.nus_uuid != (p_data[0] * 256 + p_data[1]) ||
                m_cmd_collection.nus_rx_uuid != (p_data[2] * 256 + p_data[3]) ||
                m_cmd_collection.nus_tx_uuid != (p_data[4] * 256 + p_data[5])) &&
                ( 0 <= p_data[0] && 255 >= p_data[0] && 0 <= p_data[1] && 255 >= p_data[1] &&
                  0 <= p_data[2] && 255 >= p_data[2] && 0 <= p_data[3] && 255 >= p_data[3] &&
                  0 <= p_data[4] && 255 >= p_data[4] && 0 <= p_data[5] && 255 >= p_data[5] )) {

                NRF_LOG_RAW_INFO("cmd set uuid\r\n");
                m_cmd_collection.nus_uuid    = p_data[0] * 256 + p_data[1];
                m_cmd_collection.nus_rx_uuid = p_data[2] * 256 + p_data[3];
                m_cmd_collection.nus_tx_uuid = p_data[4] * 256 + p_data[5];
                mt_reply_ok_flag = true;
                mt_flash_is_change = true;

            }
            break;
            

        case UART_CMD_IDEN_NUM:
        {
            uint8_t tmp_swap = p_data[0];
            p_data[0] = p_data[1];
            p_data[1] = tmp_swap;
            if ((memcmp(&m_cmd_collection.adv_company_id, p_data, sizeof(uint16_t)) != 0) &&  (p_len == CMD_DEV_IDENTIFY)){
                
                memcpy(&m_cmd_collection.adv_company_id, p_data, sizeof(uint16_t));
                NRF_LOG_RAW_INFO("cmd iden is %4x\r\n", m_cmd_collection.adv_company_id);
                mt_reply_ok_flag = true;
                mt_flash_is_change = true;
            }
        }
            break;   

        case UART_CMD_MAC_ADD:

            if ( p_data[0] == '?' && p_len == 1) {
                NRF_LOG_RAW_INFO("cmd mac address reply \r\n");
                mt_uart_reply(UART_REPLY_MAC);
                return;
            } else {
                break;
            }
            
        case UART_CMD_VERSON:
            if (p_data[0] == '?' && p_len == 1) {
                NRF_LOG_RAW_INFO("cmd version num reply \r\n");
                mt_uart_reply(UART_REPLY_VERSON);
                return;
            } else {
                break;
            }

        case UART_CMD_RESTE:
        {
            if (p_len == COMMAND_LEN) {
                NRF_LOG_RAW_INFO("cmd start reset \r\n");
                mt_uart_reply(UART_REPLY_OK);   
                APP_ERROR_CHECK(app_timer_start(reset_time_id, APP_TIMER_TICKS(10), NULL));
                return;
            }

            break;  
        }
           

        case UART_CMD_RESTORE:

            if (p_len == COMMAND_LEN) {
                NRF_LOG_RAW_INFO("cmd start restore \r\n");
                // mt_uart_reply(UART_REPLY_OK);
                default_param_get();
                storage_data_handler(NULL, 0);
                // while(!app_flash_update_done());  //!!! 使用while或标志位等待FDS_EVT_UPDATE回调事件成功后再复位后复位失败
                // NVIC_SystemReset();               //!!! 目前选择用手动复位，可选择自动复位与手动复位
                mt_reply_ok_flag = true;
                mt_flash_is_change = true;
            }

            break;  
        default :
            break;
    }

   
    if (mt_flash_is_change == true) {
        m_cmd_collection.magic_byte = MAGIC_DATA;
        APP_ERROR_CHECK(app_sched_event_put((void *)0, 0, storage_data_handler));
    }

    if (mt_reply_ok_flag) {
        mt_uart_reply(UART_REPLY_OK);
    } else {
        mt_uart_reply(UART_REPLY_ERROR);
    }

    // m_uart_rec_states *#StatesChange. = UART_REC_IDLE;


}


/**
 * #brief Uart command process function.
 */
static void uart_cmd_process_handler(void * p_context)  
{
    
    uint8_t  ptmp_cmd_len = m_temp_array.size ;
    uint8_t uart_cmd_data[] ={0};

    m_uart_rec_states /**#StatesChange.*/ = UART_REC_CMD_ANALY;
    NRF_LOG_RAW_INFO("m_temp_array.size =%d\r\n", m_temp_array.size);

    // NRF_LOG_HEXDUMP_INFO(m_temp_array.data, m_temp_array.size);
    if (memcmp(m_temp_array.data, (uint8_t *)COMMAND_HEAD_STRING, strlen(COMMAND_HEAD_STRING)) != 0 
        || ptmp_cmd_len < (COMMAND_MIN_LEN +1)) {
        mt_uart_reply(UART_REPLY_ERROR);
        m_temp_array.size = 0;
        NRF_LOG_RAW_INFO("TIM: HEAD ERROR %d\r\n", __LINE__);
        return;
    }

    if (!mt_uart_cmd_type_verify(m_temp_array.data + COMMAND_OFFSET) || (ptmp_cmd_len - 8- 2) < 1) {
        NRF_LOG_RAW_INFO("TIM: CMD TYPE ERROR %d\r\n", __LINE__);
        m_temp_array.size = 0;
        mt_uart_reply(UART_REPLY_ERROR);
        return;
    } 

    mt_uart_cmd_go_to(m_temp_array.data + COMMAND_MIN_LEN, ptmp_cmd_len - 8- 2);
    m_temp_array.size = 0;

    APP_ERROR_CHECK(app_fifo_flush(&m_uart_rec_fifo));
    m_uart_rec_states /**#StatesChange.*/ = UART_REC_IDLE;

}


/**
 * #brief Ble notify data function.
 * 
 * #called by  ref@uart_rec_process() - #Condition Complete Packet;
 *             ref@uart_rec_process() - #Condition Incomplete Packet;
 *             ref@on_ble_evt()       - #Case      BLE_EVT_TX_COMPLETE.
 */
static void ble_data_notify_handler(void * p_context)
{   
    if (is_device_connect() == false) {
        return;
    }
    uint32_t  err_code = ble_nus_string_send(&m_nus, m_temp_array.data, (uint16_t *)&m_temp_array.size);
    if (err_code == NRF_SUCCESS) {
        //! 成功通知一次数据，拉低一次表示对方可发送
        nrf_gpio_pin_clear(FIFO_FULL_PIN);
        NRF_LOG_WARNING("noty ok [%d]###\r\n", m_temp_array.size);

        //! 数据填入协议栈buffer成功, 清除静态数组内容.
        memset(&m_temp_array, 0, sizeof(m_temp_array));
        
        //! 上次数据填入协议栈buffer失败, 本次成功后尝试再次检索uart fifo, 查看时候有多余数据.
        if (m_uart_rec_states == UART_REC_DATA_PASS_WAIT /**#Condition.*/) {
            m_uart_rec_states /**#StatesChange.*/ = UART_REC_DATA_PASS;
            APP_ERROR_CHECK(app_sched_event_put((void *)0, 0, uart_rec_process));
        }
    }

    else 
    if (err_code == NRF_ERROR_RESOURCES) {
        NRF_LOG_WARNING("buf exce\r\n");
        m_uart_rec_states /**#StatesChange.*/ = UART_REC_DATA_PASS_WAIT;
        return;
    }

    else 
    if (err_code == BLE_ERROR_GATTS_SYS_ATTR_MISSING || 
            err_code == NRF_ERROR_INVALID_STATE) {
            NRF_LOG_WARNING("ignor error\r\n");
        //! 忽略这些由蓝牙连接带来的错误.
    } 
    
    else {
        NRF_LOG_WARNING("error check\r\n");
        APP_ERROR_CHECK(err_code);
    }
}


/**
 * #brief UART rec process function.
 *
 * #called by ref@uart_event_handle()       - #Case      - APP_UART_DATA_READY;
 * #called by ref@ble_data_notify_handler() - #Condition - NRF_SUCCESS, #Scheduled.
 */
static void  uart_rec_process(void * p_context, uint16_t data_length) 
{
    static uint8_t fifo_buffer[32];
    if (is_device_connect() == true /**< 在连接状态下，所有数据都将被视为透传数据 */) {
    // if ( 1 ) {
        m_uart_rec_states = (m_uart_rec_states == UART_REC_IDLE)? UART_REC_DATA_PASS : m_uart_rec_states;

        APP_ERROR_CHECK(app_simple_timer_stop());
        uint8_t tmp_mid_data = 0;

        while (app_fifo_get(&m_uart_rec_fifo, &tmp_mid_data) == NRF_SUCCESS) {
            m_temp_array.data[m_temp_array.size++] = tmp_mid_data;

            //! 当数据达到最大长度即尝试将数组内容全部发送出去.
            if (m_temp_array.size >= m_ble_nus_max_data_len) {
                // NRF_LOG_WARNING("size : %d \r\n",m_temp_array.size);
                ble_data_notify_handler(NULL);
                if (m_uart_rec_states == UART_REC_DATA_PASS_WAIT /**#Condition.*/) {
                    NRF_LOG_WARNING("uart wait\r\n");
                    //! DPASS WAIT 0
                    return;
                }
            }
        }


        if (m_temp_array.size > 0 && /**< 数据凑不满一整包字节的情况, 延时判断后发送. */
            m_temp_array.size < m_ble_nus_max_data_len) {
            APP_ERROR_CHECK(app_simple_timer_start(APP_SIMPLE_TIMER_MODE_SINGLE_SHOT, 
                                                   ble_data_notify_handler,
                                                   APP_TIMER_TICKS(600), 
                                                   NULL));
        }
    } else { /**< 在非连接状态下，所有数据都将被视为命令数据. */
            
        while (app_fifo_get(&m_uart_rec_fifo, fifo_buffer + m_temp_array.size) == NRF_SUCCESS) {
            APP_ERROR_CHECK(app_timer_stop(uart_clear_id));
            APP_ERROR_CHECK(app_timer_start(uart_clear_id, APP_TIMER_TICKS(200), NULL));
            if (m_temp_array.size >= 9 && m_temp_array.size <= (COMMAND_MAX_LEN-1)) {
                if (memcmp(fifo_buffer + (m_temp_array.size -1), (uint8_t *)COMMAND_END_STR, 2) == 0) {
                    APP_ERROR_CHECK(app_timer_stop(uart_clear_id));
                    APP_ERROR_CHECK(app_simple_timer_stop());
                    memcpy(m_temp_array.data, fifo_buffer, m_temp_array.size + 1);
                    APP_ERROR_CHECK(app_simple_timer_start(APP_SIMPLE_TIMER_MODE_SINGLE_SHOT, 
                                               uart_cmd_process_handler,
                                               APP_TIMER_TICKS(UART_REC_WAIT_INTERVAL), 
                                               NULL));
                } 
            } 

            m_temp_array.size ++;
        }

        /* 出口发送的配置命令大于指定长度*/
        if (m_temp_array.size > COMMAND_MAX_LEN) {
            NRF_LOG_RAW_INFO(">COMMAND_MAX_LEN stop timer\r\n");
            APP_ERROR_CHECK(app_simple_timer_stop());
            m_temp_array.size = 0;
            uart_rec_process_clr();
        }

        /* 清楚发送不足命令长度.*/
        APP_ERROR_CHECK(app_timer_stop(uart_clear_id));
        APP_ERROR_CHECK(app_timer_start(uart_clear_id, APP_TIMER_TICKS(200), NULL));


    }

    m_uart_rec_states /**#StatesChange.*/ = UART_REC_IDLE;
}


/**
 * #简介 查询串口接收处理模块当前是否繁忙.
 *
 * #返回 true   串口接收处理模块当前繁忙.
 * #返回 false  串口接收处理模块当前空闲.
 */
static inline bool is_uart_rec_in_process(void) {
    return (m_uart_rec_states == UART_REC_IDLE)? false : true;
}


/**
 * #brief  Function for handling app_uart events.
 */
void uart_event_handle(app_uart_evt_t * p_event)
{
    switch (p_event->evt_type)
    {
        /**< An event indicating that UART data has been received. 
             The data is available in the FIFO and can be fetched using @ref app_uart_get. */
        case APP_UART_DATA_READY:
        {
            uint8_t tmp_rec_data = 0;

            UNUSED_VARIABLE(app_uart_get(&tmp_rec_data));

            if (app_fifo_put(&m_uart_rec_fifo, tmp_rec_data) == NRF_SUCCESS) {
                if (is_uart_rec_in_process() == false) {
                    uart_rec_process((void *)0, 0);
                } else {
                    NRF_LOG_WARNING("uart buzy\r\n");
                    m_uart_rec_states == UART_REC_IDLE;
                    //! Busy
                    return;
                }
            } else {
                //! Rec fifo is full
                NRF_LOG_WARNING("@#$@##@%$#@^@#$%#@$%@#$%fifo full@#!$!#$@!#%#$@%@$##$@%#@$^%$#&@$#%#$%@\r\n");
                nrf_gpio_pin_set(FIFO_FULL_PIN);
                return;
            }
            break;
        }

        /**< An communication error has occured during reception. */
        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;
        
        /**< An error in the FIFO module used by the app_uart module has occured. */
        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;
        case APP_UART_TX_EMPTY:
            NRF_LOG_INFO("UART SEND COMPELETE\r\n");
            // APP_ERROR_CHECK(app_uart_flush());
            uart_rec_process_clr();
            break;

        /**< An event indicating that UART has completed transmission of all available data in the TX FIFO. */
        // case APP_UART_TX_EMPTY:
        //     //! TX_EMPTY
        //     break;

        default:
            break;
    }
}


/**
 * #brief  Function for initializing the UART module.
 */
static void uart_init(void)
{
    uint32_t err_code;
    uint32_t tmp_uart_baud_list[] = UART_BAUDRATE_LIST;

    const app_uart_comm_params_t comm_params =
    {
        .rx_pin_no    = RX_PIN_NUMBER,
        .tx_pin_no    = TX_PIN_NUMBER,
        .rts_pin_no   = RTS_PIN_NUMBER,
        .cts_pin_no   = CTS_PIN_NUMBER,
        .flow_control = APP_UART_FLOW_CONTROL_DISABLED,
        .use_parity   = false,
        .baud_rate    = tmp_uart_baud_list[m_cmd_collection.uart_baudrate]
    };

    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_event_handle,
                       APP_IRQ_PRIORITY_LOWEST,
                       err_code);
    APP_ERROR_CHECK(err_code);
}


/**
 * #简介 wstates_led_time_id timeout handler.
 */
static void led_timeout_handler(void * p_context) 
{
    bool           * p_led_states  = p_context;
    static uint8_t   stmp_led_states;

    if(*p_led_states == true) {
        nrf_gpio_pin_clear(WSTATES_PIN_NUMBER);
        stmp_led_states = false;
        APP_ERROR_CHECK(app_timer_start(wstates_led_time_id, APP_TIMER_TICKS(LED_LOW_PULSE_TIME), &stmp_led_states));
    } else {
        nrf_gpio_pin_set(WSTATES_PIN_NUMBER);
        stmp_led_states = true;
        APP_ERROR_CHECK(app_timer_start(wstates_led_time_id, APP_TIMER_TICKS(LED_HIGH_PULSE_TIME), &stmp_led_states));
    }
}


/**
 * @brief 获取uicr的数据.
 */
static void mt_uicr_get(cmd_collection_t * p_uicr_params)
{
    /* 广播连接参数 1byte广播间隔 + 1byte波特率 + 1byte连接间隔 + 1byte发射功率*/
    uint32_t * ptmp_adv_params = (uint32_t *)0x10001080;
    p_uicr_params->adv_interval_level = ((*ptmp_adv_params) >> 24) & 0xFF;
    p_uicr_params->uart_baudrate      = ((*ptmp_adv_params) >> 16) & 0xFF;
    p_uicr_params->con_interval_level = ((*ptmp_adv_params) >> 8) & 0xFF;
    p_uicr_params->tx_power           = ((*ptmp_adv_params) >> 0) & 0xFF;
    NRF_LOG_RAW_INFO("adv params adv_itv[%x], uar_bde[%x], con_ite[%x], tx_power[%x]\r\n", p_uicr_params->adv_interval_level,
        p_uicr_params->uart_baudrate, p_uicr_params->con_interval_level, p_uicr_params->tx_power);


    /* 2byte 产品识别码 + 2 byete NUS uuid*/
    uint32_t * ptmp_nus_uuid_h = (uint32_t *)0x10001084;
    p_uicr_params->dummy    = (((*ptmp_nus_uuid_h) >> 24) & 0xFF) *256 + (((*ptmp_nus_uuid_h) >> 16) & 0xFF);
    p_uicr_params->nus_uuid = (((*ptmp_nus_uuid_h) >> 8) & 0xFF) *256 + (((*ptmp_nus_uuid_h) >> 0) & 0xFF);
    NRF_LOG_RAW_INFO("adv_company_id %4x\r\n", p_uicr_params->dummy);
    NRF_LOG_RAW_INFO("nus uuid %4x \r\n",p_uicr_params->nus_uuid );


    /* 4byte NUS tx +rx uuid*/
    uint32_t * ptmp_nus_uuid_l = (uint32_t *)0x10001088;
    p_uicr_params->nus_rx_uuid  = (((*ptmp_nus_uuid_l) >> 24) & 0xFF) *256 + (((*ptmp_nus_uuid_l) >> 16) & 0xFF);
    p_uicr_params->nus_tx_uuid  = (((*ptmp_nus_uuid_l) >> 8) & 0xFF) *256 + (((*ptmp_nus_uuid_l) >> 0) & 0xFF);
    NRF_LOG_RAW_INFO("nus tx rx uuid %4x, %4x\r\n", p_uicr_params->nus_tx_uuid, p_uicr_params->nus_rx_uuid);
 

    /* 4byte maddress */
    uint32_t * ptmp_mac_addr_h = (uint32_t *)0x1000108c;
    p_uicr_params->mac_address[0] = (((*ptmp_mac_addr_h) >> 24) & 0xFF);
    p_uicr_params->mac_address[1] = (((*ptmp_mac_addr_h) >> 16) & 0xFF);
    p_uicr_params->mac_address[2] = (((*ptmp_mac_addr_h) >> 8) & 0xFF);
    p_uicr_params->mac_address[3] = (((*ptmp_mac_addr_h) >> 0) & 0xFF);


    /* 2byte maddress + 2byte保留*/
    uint32_t * ptmp_mac_addr_l = (uint32_t *)0x10001090;
    p_uicr_params->mac_address[4] = (((*ptmp_mac_addr_l) >> 24) & 0xFF);
    p_uicr_params->mac_address[5] = (((*ptmp_mac_addr_l) >> 16) & 0xFF);
    p_uicr_params->adv_name.data_length = ((*ptmp_mac_addr_l) >> 8) & 0xFF;
    /*p_uicr_params->adv_data.data_length = ((*ptmp_mac_addr_l) >> 0) & 0xFF;*/
    NRF_LOG_RAW_INFO("mac address");
    NRF_LOG_HEXDUMP_INFO(p_uicr_params->mac_address,  6);
    NRF_LOG_RAW_INFO("reserve %d\r\n", p_uicr_params->adv_name.data_length);


    /* 16byte 设备名称*/
    uint32_t * ptmp_dev_name_1 = (uint32_t *)0x10001094;
    uint32_t * ptmp_dev_name_2 = (uint32_t *)0x10001098;
    uint32_t * ptmp_dev_name_3 = (uint32_t *)0x1000109c;
    uint32_t * ptmp_dev_name_4 = (uint32_t *)0x100010a0;

    memcpy(p_uicr_params->adv_name.data_array,        ptmp_dev_name_1, 4);
    memcpy(p_uicr_params->adv_name.data_array + 4,    ptmp_dev_name_2, 4);
    memcpy(p_uicr_params->adv_name.data_array + 8,    ptmp_dev_name_3, 4);
    memcpy(p_uicr_params->adv_name.data_array + 12,   ptmp_dev_name_4, 4);
    array_reserve(p_uicr_params->adv_name.data_array, 4);
    array_reserve(p_uicr_params->adv_name.data_array + 4, 4);
    array_reserve(p_uicr_params->adv_name.data_array + 8, 4);
    array_reserve(p_uicr_params->adv_name.data_array + 12, 4);
    NRF_LOG_RAW_INFO("device name ");
    NRF_LOG_HEXDUMP_INFO(p_uicr_params->adv_name.data_array,  16);
}


/**
 * #brief Function for default_param: baudrate，company_id，interval_level，interval_level，tx_power
 */
static void default_param_get(void)
{
    uint8_t mac_add[]= {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
    cmd_collection_t uicr_data ;
    /* 获取uicr 数据. */
    mt_uicr_get(&uicr_data);

#if SRPDATA_DEFAULT
    m_cmd_collection.adv_data.data_length  = strlen(DEFAULT_MANUF_DATA);
    memcpy(m_cmd_collection.adv_data.data_array, (uint8_t *)DEFAULT_MANUF_DATA, strlen(DEFAULT_MANUF_DATA));
#else
    m_cmd_collection.adv_data.data_length = 0;
#endif


    /* 广播名称填充 */
    if (uicr_data.adv_name.data_length >= 0 && uicr_data.adv_name.data_length <= 16) {
        memcpy(m_cmd_collection.adv_name.data_array, &(uicr_data.adv_name.data_array), uicr_data.adv_name.data_length);
        m_cmd_collection.adv_name.data_length  = uicr_data.adv_name.data_length;
    } else {
        m_cmd_collection.adv_name.data_length  = strlen(DEFAULT_DEVICE_NAME);
        memcpy(m_cmd_collection.adv_name.data_array, (uint8_t *)DEFAULT_DEVICE_NAME, strlen(DEFAULT_DEVICE_NAME));
    }

    /* 波特率填充. */
    if (uicr_data.uart_baudrate >= 0 && uicr_data.uart_baudrate <= 4) {
        m_cmd_collection.uart_baudrate = uicr_data.uart_baudrate;
    } else {
        m_cmd_collection.uart_baudrate = DEFAULT_UART_BAUD;
    }

    /* service uuid 即产品识别码 填充.*/
    if (((uicr_data.dummy >> 8) & 0xff) != 0xff || (uicr_data.dummy & 0xff)!= 0xff) {
        m_cmd_collection.dummy= uicr_data.dummy;
    } else {
        m_cmd_collection.dummy = SERVICE_UUID;
    }

    m_cmd_collection.adv_company_id = DEFAULT_COMPANY_ID;
    
    /* 广播间隔填充.*/
    if (uicr_data.adv_interval_level >= 1 && uicr_data.adv_interval_level <= 20) {
        m_cmd_collection.adv_interval_level = uicr_data.adv_interval_level;
    } else {
        m_cmd_collection.adv_interval_level = DEFAULT_ADV_INTERVAL;
    }

    /* 连接间隔填充. */
    if (uicr_data.con_interval_level >= 2 && uicr_data.con_interval_level <= 100) {
        m_cmd_collection.con_interval_level = uicr_data.con_interval_level;
    } else {
        m_cmd_collection.con_interval_level = DEFAULT_CONN_INTERVAL;
    }

    /* 发射功率填充. */
    if (uicr_data.tx_power >= 0 && uicr_data.tx_power <= 7) {
        m_cmd_collection.tx_power = uicr_data.tx_power;
    } else {
        m_cmd_collection.tx_power = DEFAULT_TX_POWER;
    }

    /* mac地址填充. */
    if (memcmp(uicr_data.mac_address, mac_add, MAC_ADDR_LEN) == 0) {
       static ble_gap_addr_t  tmp_mac_data;
        APP_ERROR_CHECK(sd_ble_gap_addr_get(&tmp_mac_data));
        NRF_LOG_HEXDUMP_INFO(tmp_mac_data.addr, MAC_ADDR_LEN);
        memcpy(m_cmd_collection.mac_address, tmp_mac_data.addr, MAC_ADDR_LEN);
    } else {
        memcpy(m_cmd_collection.mac_address, uicr_data.mac_address, MAC_ADDR_LEN);
        array_reserve(m_cmd_collection.mac_address, MAC_ADDR_LEN); 
    }

    /* uuid 填充. */
    if (uicr_data.nus_uuid == 0xFFFF){
        m_cmd_collection.nus_uuid = DEFAULT_BLE_UUID_NUS_SERVICE;
    } else {
        m_cmd_collection.nus_uuid = uicr_data.nus_uuid;
    }


    if (uicr_data.nus_tx_uuid == 0xFFFF) {
        m_cmd_collection.nus_tx_uuid = DEFAULT_BLE_UUID_NUS_TX_CHARACTERISTIC;
    } else {
        m_cmd_collection.nus_tx_uuid = uicr_data.nus_tx_uuid;
    }

    if (uicr_data.nus_rx_uuid == 0xFFFF) {
        m_cmd_collection.nus_rx_uuid = DEFAULT_BLE_UUID_NUS_RX_CHARACTERISTIC;
    } else {
        m_cmd_collection.nus_rx_uuid = uicr_data.nus_rx_uuid;
    }

    NRF_LOG_RAW_INFO(" uuid%4x  tx uuid %4x   rx uuid %4x", m_cmd_collection.nus_uuid, m_cmd_collection.nus_tx_uuid,m_cmd_collection.nus_rx_uuid);
}


/**
 * #brief ...
 */
static void init_fds_evt_handler(fds_evt_t const * const p_fds_evt)
{
    switch (p_fds_evt->id)
    {
        case FDS_EVT_INIT:
            if (p_fds_evt->result == FDS_SUCCESS) {
                NRF_LOG_WARNING("fds init sus\r\n");
                m_fds_init_flag = true;
            } else {
                // NRF_LOG_ERROR("[%04d] FDS init failed. \r\n", __LINE__, p_fds_evt->result);
            }
            break;

        case FDS_EVT_WRITE:
            if (p_fds_evt->result == FDS_SUCCESS) {
                m_cmd_collection.magic_byte = MAGIC_DATA;
                NRF_LOG_WARNING("fds write sus\r\n");
            } else {
                NRF_LOG_ERROR("fds write failed %d. \r\n", p_fds_evt->result);
            }
            break;

        case FDS_EVT_UPDATE:
            if (p_fds_evt->result == FDS_SUCCESS) {
                NRF_LOG_WARNING("fds update sus\r\n");
                m_update_in_queue -= 1;
            } else {
                //!
            }
            break;

        case FDS_EVT_GC:
            if (p_fds_evt->result == FDS_SUCCESS) {
                NRF_LOG_WARNING("fds gc sus\r\n");
            } else {
                //!
            }
            break;

        default:
            break;
    }
}


/**
 * #brief fsd init ....
 */
static void fds_data_init(void)
{
    fds_flash_record_t  flash_record;
	fds_record_desc_t   record_desc;
    fds_find_token_t    ftok;
    
    //! Flash上电初始化. 
    APP_ERROR_CHECK(fds_register(init_fds_evt_handler));
    APP_ERROR_CHECK(fds_init());
    do {
        app_sched_execute();
    } while (m_fds_init_flag == false);

    //! 检索数据
    memset(&ftok, 0x00, sizeof(fds_find_token_t));
    uint32_t err_code = fds_record_find(FILE_ID_MS_HANDLE, REC_KEY_MS_HANDLE, &record_desc, &ftok);
    if ( err_code == FDS_SUCCESS) {
        NRF_LOG_WARNING("Record found\r\n");
        APP_ERROR_CHECK(fds_record_open(&record_desc, &flash_record));
		memcpy(&m_cmd_collection, flash_record.p_data, sizeof(m_cmd_collection));
		APP_ERROR_CHECK(fds_record_close(&record_desc));
	} else if (err_code == FDS_ERR_NOT_FOUND) {
        NRF_LOG_WARNING("not found.get default_param.\r\n");
        default_param_get();
        storage_data_handler(NULL, 0);
    } else {
        APP_ERROR_CHECK(err_code);
    }
}


/**
 * #brief Function for initializing the Advertising functionality.
 */
static void advertising_start(void)
{
    ble_gap_adv_params_t  adv_params;
    ble_gap_addr_t m_ble_gap_addr; 
    int8_t                tx_pwr[] = RADIO_TX_POWER_LIST;

    adv_params.type        = BLE_GAP_ADV_TYPE_ADV_IND;
    adv_params.p_peer_addr = NULL;
    adv_params.fp          = BLE_GAP_ADV_FP_ANY;
    adv_params.timeout     = DEFAULT_ADV_TIMEOUT;
    adv_params.interval    = MSEC_TO_UNITS(m_cmd_collection.adv_interval_level * 100, UNIT_0_625_MS);
    
    memcpy(m_ble_gap_addr.addr, m_cmd_collection.mac_address, MAC_ADDR_LEN);  
 
    m_ble_gap_addr.addr_type = ((m_ble_gap_addr.addr[5] & 0xC0) == 0xC0) ? BLE_GAP_ADDR_TYPE_RANDOM_STATIC : BLE_GAP_ADDR_TYPE_PUBLIC;
    
    APP_ERROR_CHECK(sd_ble_gap_addr_set(&m_ble_gap_addr));
    APP_ERROR_CHECK(sd_ble_gap_tx_power_set(tx_pwr[m_cmd_collection.tx_power]));
    APP_ERROR_CHECK(sd_ble_gap_adv_start(&adv_params, CONN_CFG_TAG));

    app_led_handle(LED_BLE_ADVERTISING, WSTATES_PIN_NUMBER);
}


/**
 * #brief Function for initializing the Advertising functionality.
 */
static void advertising_init(void)
{
    ble_advdata_t  tmp_advdata;
    ble_advdata_t  tmp_srpdata;
    uint8_t tmp_dev_mac[6];

    memcpy(tmp_dev_mac, m_cmd_collection.mac_address, MAC_ADDR_LEN);
    array_reserve(tmp_dev_mac, MAC_ADDR_LEN); 

    
    /* 服务数据 主要包含mac地址*/
    uint8_array_t service_data  = { .size = sizeof(tmp_dev_mac),
                                    .p_data = (uint8_t *)&tmp_dev_mac};
                                                                        
    ble_advdata_service_data_t servce = {.service_uuid = m_cmd_collection.dummy,
                                         .data = service_data};
    ble_uuid_t complete_uuids;
    complete_uuids.type = BLE_UUID_TYPE_BLE;    
    complete_uuids.uuid = m_cmd_collection.dummy;

    /* 厂商自定义数据 主要存放用户自定义广播数据.*/
    uint8_array_t  tmp_manuf_data = { .size = m_cmd_collection.adv_data.data_length, 
                                      .p_data = m_cmd_collection.adv_data.data_array };
    ble_advdata_manuf_data_t  tmp_manuf = { .company_identifier = m_cmd_collection.adv_company_id, 
                                            .data = tmp_manuf_data };

    /* 广播内容填充. */                                   
    memset(&tmp_advdata, 0, sizeof(tmp_advdata));
    tmp_advdata.flags                 = DEFAULT_ADV_FLAGS;
    tmp_advdata.include_appearance    = false;
    tmp_advdata.p_service_data_array  = &servce;
    tmp_advdata.service_data_count = 0x01;

    /* 2 + 17 + 9 =28byte 装不下 commplete_uuid*/
    /* tmp_advdata.uuids_complete.p_uuids = &complete_uuids;
    tmp_advdata.uuids_complete.uuid_cnt = 0x01;*/
    tmp_advdata.name_type = BLE_ADVDATA_FULL_NAME;

    /* 扫描回复包里放广播数据,最大27byte. */
    memset(&tmp_srpdata, 0, sizeof(tmp_srpdata));
    tmp_srpdata.p_manuf_specific_data = &tmp_manuf;

    NRF_LOG_RAW_INFO("no srdv start\r\n");
    if (m_cmd_collection.adv_data.data_length == 0) {
        NRF_LOG_RAW_INFO("no srdv\r\n");
        APP_ERROR_CHECK(ble_advdata_set(&tmp_advdata, NULL));
    } else {
        APP_ERROR_CHECK(ble_advdata_set(&tmp_advdata, &tmp_srpdata));
    }
}


/**
 * #brief 看门狗中断与看门狗手动喂狗
 * #note 在这个中断内最长可以有2个低频周期(2/32765(s))的时间，然后便会复位.
 */
static void wdt_event_handler(void) 
{
    NRF_LOG_RAW_INFO("wdt reset\r\n");
    // [todo]
}

static void mt_power_off(void * p_event_data, uint16_t event_size)
{
    APP_ERROR_CHECK(sd_power_system_off());
}


/**
 * #brief Function for putting the chip into sleep mode.
 *
 * #note This function will not return.
 */
static void sleep_mode_enter(void)
{
    //! Prepare wakeup buttons.
    NRF_GPIO->PIN_CNF[SLEEP_PIN_NUMBER] = (GPIO_PIN_CNF_SENSE_Low     << GPIO_PIN_CNF_SENSE_Pos) 
                                        | (GPIO_PIN_CNF_DRIVE_S0S1    << GPIO_PIN_CNF_DRIVE_Pos) 
                                        | (NRF_GPIO_PIN_NOPULL        << GPIO_PIN_CNF_PULL_Pos) 
                                        | (GPIO_PIN_CNF_INPUT_Connect << GPIO_PIN_CNF_INPUT_Pos) 
                                        | (GPIO_PIN_CNF_DIR_Input     << GPIO_PIN_CNF_DIR_Pos);
    //! close......
    if(nrf_gpio_pin_read(BTDATA_PIN_NUMBER) == 0){
        app_uart_close();
    }
    app_led_handle(LED_SLEEP, WSTATES_PIN_NUMBER);
    nrf_gpio_cfg_output(BTDATA_PIN_NUMBER);
    nrf_gpio_cfg_output(RESET_PIN_NUMBER);
    nrf_gpio_pin_set(BTDATA_PIN_NUMBER);
    nrf_gpio_pin_set(RESET_PIN_NUMBER);


   NRF_LOG_RAW_INFO( "start sllepp\r\n");
    //! Go to system-off mode (this function will not return; wakeup will cause a reset).
    APP_ERROR_CHECK(app_sched_event_put((void *)0, 0, sd_power_system_off));
    //! 可用最低优先级，防止出现错误 <NRF_ERROR_SOC_POWER_OFF_SHOULD_NOT_RETURN>
}


/**
 * #brief Function for button handler
 */
static void button_event_handler(uint8_t pin_no, uint8_t button_action)
{
    if (button_action == APP_BUTTON_PUSH)	
    {
        switch (pin_no)
        {
            case SLEEP_PIN_NUMBER:
				break;

            case BTDATA_PIN_NUMBER:
                    uart_init();
                
                break;

			default:
				break;
		}
	}
	
    if (button_action == APP_BUTTON_RELEASE)	
    {
        switch (pin_no)
        {
            case SLEEP_PIN_NUMBER:
				sleep_mode_enter();
				break;

            case BTDATA_PIN_NUMBER:
                    APP_ERROR_CHECK(app_uart_close());


                break;
 
			default:
				break;
		}
	}
}


/**
 * #简介 上电引脚检测
 */
static void default_pin_check(void)
{
    if(nrf_gpio_pin_read(BTDATA_PIN_NUMBER) == 0) {
        uart_init();
    }

    if(nrf_gpio_pin_read(SLEEP_PIN_NUMBER) != 0) {  //! 或!= 0（悬空与拉高）
        sleep_mode_enter();
    }
}


/**
 * #简介 定时器循环喂狗.
 */
static void wdt_timeout_handler(void * p_context) {

    nrf_drv_wdt_channel_feed(m_channel_id);
}

/**
 * #简介 定时器循环喂狗.
 */
static void reset_timeout_handler(void * p_context) {

    NVIC_SystemReset();
}

static void uart_clear_handler(void * p_context) 
{
    // if (m_temp_array.size < 9) {
        m_temp_array.size = 0;
        uart_rec_process_clr();
    // } 

}




/**
 * #brief Function for gpiote:1.SLEEP 2.WSTATES 3.RST 4.BTDATA
 */
static void gpio_init(void)
{
    nrf_gpio_cfg_output(WSTATES_PIN_NUMBER);
    nrf_gpio_cfg_output(FIFO_FULL_PIN);
    nrf_gpio_pin_clear(FIFO_FULL_PIN);

    static app_button_cfg_t buttons[] =
    {
       {SLEEP_PIN_NUMBER,  APP_BUTTON_ACTIVE_LOW, NRF_GPIO_PIN_NOPULL, button_event_handler},
       {BTDATA_PIN_NUMBER, APP_BUTTON_ACTIVE_LOW, NRF_GPIO_PIN_NOPULL, button_event_handler}
    };
    
    APP_ERROR_CHECK(app_button_init(buttons, sizeof(buttons) / sizeof(buttons[0]), BUTTON_DETECTION_DELAY));
	APP_ERROR_CHECK(app_button_enable());
	
}

/**
 * @brief LOG初始化.
 */
static void log_init(void)
{
    APP_ERROR_CHECK(NRF_LOG_INIT(NULL));
    NRF_LOG_DEFAULT_BACKENDS_INIT();
}


/**
 * #brief 基本要素初始化.
 */
static void basic_init(void)
{
    //! 上电延时一段时间等待电源稳定.
    nrf_delay_ms(400);

    log_init();
    // Initialize.
    APP_ERROR_CHECK(app_timer_init());
    APP_SCHED_INIT(SCHED_MAX_EVENT_DATA_SIZE, SCHED_QUEUE_SIZE);

    //! 看门狗初始化.
    nrf_drv_wdt_config_t config = NRF_DRV_WDT_DEAFULT_CONFIG;
    APP_ERROR_CHECK(nrf_drv_wdt_init(&config, wdt_event_handler));
    APP_ERROR_CHECK(nrf_drv_wdt_channel_alloc(&m_channel_id));

    APP_ERROR_CHECK(app_simple_timer_init());
    APP_ERROR_CHECK(app_timer_create(&wstates_led_time_id, APP_TIMER_MODE_SINGLE_SHOT, led_timeout_handler));

    APP_ERROR_CHECK(app_timer_create(&wdt_time_id, APP_TIMER_MODE_REPEATED, wdt_timeout_handler));

    APP_ERROR_CHECK(app_timer_create(&reset_time_id, APP_TIMER_MODE_SINGLE_SHOT, reset_timeout_handler));

    APP_ERROR_CHECK(app_timer_create(&uart_clear_id, APP_TIMER_MODE_SINGLE_SHOT, uart_clear_handler));

}


/**
 * #brief 协议栈与flash等初始化.
 */
static void ble_init(void)
{
    gpio_init();
    ble_stack_init();
    fds_data_init();
    gap_params_init();
    gatt_init();
    conn_params_init();
    services_init();
}


/**
 * #brief 硬件初始化
 */
static void hal_init(void)
{
    default_pin_check();
    // uart_init();
    APP_ERROR_CHECK(app_fifo_init(&m_uart_rec_fifo, m_rec_buf, sizeof(m_rec_buf)));
    uart_rec_process_clr();
}


/** 
 * @brief Function for the Power manager.
 */
static void power_manage(void)
{
    ret_code_t err_code = sd_app_evt_wait();
    APP_ERROR_CHECK(err_code);
}


/**
 * #brief Application main function.
 */
int main(void)
{
    basic_init();
    ble_init();
    hal_init();


    uint32_t tmp_reset_reason = 0;
     /* 获取程序上电复位原因.*/ 
    APP_ERROR_CHECK(sd_power_reset_reason_get(&tmp_reset_reason));
    NRF_LOG_INFO("[%04d] Power on reset reason is: %08x.\r\n", __LINE__, tmp_reset_reason);

    APP_ERROR_CHECK(sd_power_dcdc_mode_set(NRF_POWER_DCDC_ENABLE));
    /* 主动清除reset_reason寄存器的内容.*/ 
    tmp_reset_reason = 0xFFFFFFFF;
    APP_ERROR_CHECK(sd_power_reset_reason_clr(tmp_reset_reason));
    
    advertising_init();
    advertising_start();
    nrf_drv_wdt_enable();
    APP_ERROR_CHECK(app_timer_start(wdt_time_id, APP_TIMER_TICKS(WDT_TIMEOUT_TIME), NULL));

    NRF_LOG_INFO("main\r\n");
    // Enter main loop.
    for (;;) {
        nrf_drv_wdt_feed();
        app_sched_execute();
        if (NRF_LOG_PROCESS() == false)
        {
            power_manage(); 
        }
        
    }
}